\\ Copyright 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("../geom.gp");
read("../geom-inertia.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     \\quit(1)
     ,
     print("ok "check_count" "name));
}

matrix_sum(m)=vecsum(sum(i=1,matsize(m)[2],m[,i]));

nearly_equal_epsilon = 1e-15;
nearly_equal(x,y, epsilon=nearly_equal_epsilon) = {
  x=abs(x-y);
  if(type(x)=="t_VEC" || type(x)=="t_COL",x=vecsum(x));
  if(type(x)=="t_MAT",x=matrix_sum(x));
  if(type(x)=="t_QUAD" || type(x)=="t_COMPLEX",x=real(x)+imag(x));
  x < epsilon;
}
check_nearly(got,want,name="") =
{
  check_count++;
  if(! nearly_equal(got,want),
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}


\\-----------------------------------------------------------------------------
\\ Rotation

M =[  Ix,  -Ixy,  -Ixz; \
    -Iyx,  Iy,   -Iyz; \
    -Izx, -Izy,   Iz ]
geom_inertia_rotate_xydir(M,[x,y])
geom_inertia_rotate_xydir(M,[1,0])

XY = [0,1,0; \
      -1,0,0; \
      0,0,1]
XZ = [0,0,1; \
      0,1,0; \
      -1,0,0]

check_exact(geom_inertia_rotate_xydir(M,[0,1]), XY*M*XY^-1);

XZ*M*XZ^-1

Rxy = [ x,  y, 0; \
       -y, x, 0; \
        0,  0, 1 ];
Rxz = [ x,  0, z; \
        0,  1, 0; \
       -z,  0, x ];

\\ check_exact(geom_inertia_rotate_xydir(M,[x,y]), \
\\             Rxy*M*Rxy^-1 * [1/(x^2+y^2),0,0; 0,1/(x^2+y^2),0; 0,0,1]);
\\Rxy*M*Rxy^-1

geom_inertia_rotate_pdir(M,p) =
{
  my(x = geom_point_x(p),
     y = geom_point_y(p),
     z = geom_point_z(p),
     n = geom_sqrt_as_quad(simplify(x^2) + simplify(y^2)),
     xn = x/n,
     zn = z/n,
     P=[ x*z/n, -y/n, x;
        y*z/n,  x/n, y;
        -n, 0, z ]);
  P*M*mattranspose(P);
}
geom_inertia_rotate_pdir(M,p) =
{
  my(x = geom_point_x(p),
     y = geom_point_y(p),
     n = 'n); \\ geom_sqrt_as_quad(simplify(x^2) + simplify(y^2)));
  if(n,
     my(xn = x/n,
        yn = y/n);
     M = [ xn,  yn, 0;
          -yn, xn, 0;
          0,  0, 1 ]
        * M *
        [ xn, -yn,  0;
         yn,  xn,  0;
         0,   0,  1 ]);

  h = n; \\ geom_abs([x,y]);
  my(z = geom_point_z(p),
     n = 'm, \\ geom_sqrt_as_quad(simplify(x^2) + simplify(y^2) + simplify(z^2)),
     hn = h/n,
     zn = z/n);
  M = [ hn,  0, zn;
       0,    1, 0;
       -zn,  0, hn ]
     * M *
     [ hn,  0,  -zn;
        0,  1,  0;
       zn,  0,  hn ];
  M;
}

geom_inertia_of_point([3,4,5])
geom_inertia_of_point([4,-3,5])
geom_inertia_rotate_xydir(geom_inertia_of_point([3,4,5]), [2,2,0])
geom_inertia_rotate_pdir(geom_inertia_of_point([3,4,5]), [2,2,0])

check_exact(geom_inertia_rotate_pdir(geom_inertia_of_point([3,4,5]), [0,0,1]),\
            geom_inertia_of_point([5,4,-3]));


geom_inertia_I_at_pdir(M,p) = \
  geom_inertia_Ix(geom_inertia_rotate_pdir(M,p));

4^2+5^2
geom_inertia_I_at_pdir(geom_inertia_of_point([3,4,5]),[1,0,0])
geom_inertia_I_at_pdir(geom_inertia_of_point([3,4,5]),[0,1,0])
geom_inertia_I_at_pdir(geom_inertia_of_point([3,4,5]),[0,0,1])
geom_inertia_I_at_pdir(geom_inertia_of_point([3,4,5]),[3,4,5])


M =[  Ix,  -Ixy,  -Ixz; \
    -Ixy,  Iy,   -Iyz; \
    -Ixz, -Iyz,   Iz ]
i=geom_inertia_I_at_pdir(M,['x;'y;'z])
substpol(i, m^2, x^2+y^2+z^2)  * (x^2 + y^2 + z^2)

\\ Ix*x^2 + Iy*y^2 + Iz*z^2 - 2*(x*y*Ixy + x*z*Ixz + y*z*Iyz)




\\ c + x^2 (1-c)   xy(1-c) - zs    xz(1-c) + ys
\\                 c + y^2(1-c)    yz(1-c) - xs
\\                                 c + z^2(1-c)
\\
\\ c=0 s=1 is
\\     x^2    xy - z     xz + y 
\\              y^2      yz - x 
\\                         z^2       
\\
\\ P = [  xz/h,  -y/h,  x;
\\        yz/h,  x/h,   y;
\\         -h,    0     z ]


quit


\\-----------------------------------------------------------------------------

{
  M =[  Ix,  -Ixy,  -Ixz;
      -Iyx,  Iy,   -Iyz; 
      -Izx, -Izy,   Iz ];
  [ x/s,  y/s, 0;
    -y/s, x/s, 0;
    0,  0, 1 ]
  * M *
  [ x/s, -y/s,  0;
    y/s,  x/s,  0;
     0,   0,  1 ] *s^2

\\ [Ix*x^2 + (-Ixy - Iyx)*y*x + Iy*y^2,  -Ixy*x^2 + (-Ix + Iy)*y*x + Iyx*y^2,
\\   -s*x*Ixz - s*y*Iyz]
\\ 
\\ [-Iyx*x^2 + (-Ix + Iy)*y*x + Ixy*y^2,  Iy*x^2 + (Ixy + Iyx)*y*x + Ix*y^2,
\\   -s*x*Iyz + s*y*Ixz]
\\ 
\\ [-s*x*Izx - s*y*Izy,   -s*x*Izy + s*y*Izx,   s^2*Iz]

}
quit


\\-----------------------------------------------------------------------------
\\ Area of Triangle

geom_INTERNAL_cross_product(1+I, I-1 )
geom_INTERNAL_cross_product(2+I, (1+3*I)*2 )
\\ norm(2+I)
\\ norm(1+3*I)

a3=0
b3=0
v=geom_INTERNAL_cross_product([a1,a2,a3],[b1,b2,b3])
geom_norm(v)

geom_abs(1/2*geom_INTERNAL_cross_product(2, 2 + 2*I ))

geom_area_of_triangle(2+I, 1+3*I)

area_of_triangle_by_conj(p1,p2) =
{
  p1 -= p3;
  p2 -= p3;
  1/2 * geom_point_y(conj(p1)*p2);
}


quit

\\-----------------------------------------------------------------------------
\\ Triangle at Centroid

\\ old mass 1:
  \\ my(a=geom_point_x(z1), b=geom_point_y(z1), 
  \\    c=geom_point_x(z2), d=geom_point_y(z2));
  \\ geom_inertia_of_Ix_Iy_Ixy(
  \\   1/6*( b^2 + d*b + d^2 ),
  \\   1/6*( a^2 + c*a + c^2 ),
  \\   1/6*( a*(b + 1/2*d) + c*(1/2*b + d) ));

\\ old mass equal to area:
  \\ my(a=geom_point_x(z1), b=geom_point_y(z1), 
  \\    c=geom_point_x(z2), d=geom_point_y(z2));
  \\ geom_inertia_of_Ix_Iy_Ixy(
  \\   1/12*(  a*b^2*d + a*b*d^2 - b*c*d^2 + a*d^3  - b^2*c*d - b^3*c ),
  \\   1/12*(- b*a^2*c - b*a*c^2 + a*d*c^2 - b*c^3  + a^2*d*c + a^3*d ),
  \\                     
  \\   1/12*(  a*d^2*c + 1/2*a^2*d^2 + a^2*b*d 
  \\         - b*c^2*d - 1/2*b^2*c^2 - b^2*a*c ));

\\ old polygon by quadrants
  \\ v = geom_inertia__polygon_split_into_quadrants(v);
  \\ sum(i=1,#v,
  \\       geom_inertia_wedge_in_one(v[i], v[(i%#v) + 1]));



p1 = 'x1 + 'y1*I
p2 = 'x2 + 'y2*I
\\ (p1+p2+p3)/3 = 0
p3 = -p1 -p2
p3 = 'x3 + 'y3*I
v = [p1,p2,p3];
c = geom_centroid_of_triangle(p1,p2,p3);
print("centroid "c);
geom_inertia_of_polygon(v)

print("unit mass");
Itriangle = geom_inertia_of_polygon(v) / geom_area_of_polygon(v)  * 6
\\geom_inertia_of_point(geom_centroid_of_triangle(p1,p2))

geom_inertia_of_polygon(v) / geom_area_of_polygon(v)  * 6 \
   - geom_inertia_of_point((x1+x2)+('y1+'y2)*I)

print("of midpoints");
Imids = geom_inertia_of_point(p1+p2)/2 \
+geom_inertia_of_point(p2+p3)/2 \
+geom_inertia_of_point(p3+p1)/2
check_exact(Imids, Itriangle);

\\ print("of vertices");
\\ Iverts = geom_inertia_of_point(p1) \
\\ +geom_inertia_of_point(p2) \
\\ +geom_inertia_of_point(p3)
\\ 
\\ print("of lines");
\\ Iverts = (  geom_inertia_of_segment_m1(p1,p2) \
\\           + geom_inertia_of_segment_m1(p2,p3) \
\\           + geom_inertia_of_segment_m1(p3,p1) ) *3/2

quit

\\-----------------------------------------------------------------------------
\\ Line Segment Endpoints

p1=['a;'b;'c]
p2=['d;'e;'f]
M=geom_inertia_of_segment_m1(p1,p2)

geom_inertia_of_point(p1) + geom_inertia_of_point(p2)

quit

\\-----------------------------------------------------------------------------
\\ Line Segment 3D

\\ len = abs(z1-z2)

\\ t=0,1   z = z1*t + (1-t)*z2  = z2 + (z1-z2)*t
\\           = (c + (a-c)*t) + ((b-d)*t + d)*I
\\ poly = intformal(((b-d)*t + d)^2, t); subst(poly, t,1) - subst(poly, t,0)
\\ poly = intformal((c + (a-c)*t)^2, t); subst(poly, t,1) - subst(poly, t,0)
\\ poly = intformal(-2*(c + (a-c)*t)*((b-d)*t + d), t); subst(poly, t,1) - subst(poly, t,0)

\\ length of dt is hypot( (a-c)*dt, (b-d)*dt )
\\ total length hypot( (a-c), (b-d) )
\\   
\\ = [ z2 + (z1-z2)*1/2*t^2 ] at t=0,1
\\ = z2 + (z1-z2)*1/2 - z2
\\ = 1/2*(z1-z2)

\\ line segment mass 1 in XY plane
  \\ my(a=geom_point_x(p1), b=geom_point_y(p1), 
  \\    c=geom_point_x(p2), d=geom_point_y(p2));
  \\ geom_inertia_of_Ix_Iy_Ixy(1/3*( b^2 + d*b + d^2 ),             \\ Ix
  \\                           1/3*( a^2 + a*c + c^2 ),             \\ Iy
  \\                           1/6*( a*d + b*c + 2*b*a + 2*d*c ));  \\ Ixy


\\ t=0 to 1 along length
inertia_of_intformal_t1(pt) =
{
  my(func=poly-> poly=intformal(poly,'t); subst(poly,'t,1) - subst(poly,'t,0));
  [ func(geom_point_y(pt)^2 + geom_point_z(pt)^2), 
   -func(geom_point_y(pt)*geom_point_x(pt)),
   -func(geom_point_z(pt)*geom_point_x(pt));
   
   -func(geom_point_x(pt)*geom_point_y(pt)),
   func(geom_point_x(pt)^2 + geom_point_z(pt)^2), 
   -func(geom_point_z(pt)*geom_point_y(pt));
   
   -func(geom_point_x(pt)*geom_point_z(pt)),
   -func(geom_point_y(pt)*geom_point_z(pt)),
   func(geom_point_x(pt)^2 + geom_point_y(pt)^2) ];
}
\\ M=inertia_of_intformal_t1(pt)

inertia_of_segment(p1,p2) = inertia_of_intformal_t1(p1 + 't*(p2-p1));

p1=['a;'b;'c]
p2=['d;'e;'f]
M=inertia_of_segment(p1,p2)
\\ \\
\\ pt = 



print("subst down");
subst(subst(M,'c,0),'f,0)

print("segm");
geom_inertia_of_segment_m1(['a;'b], ['d;'e])

pdelta = p2-p1
pmid = (p1+p2)/2
\\ M=inertia_of_segment(0,pdelta)
centroid_of_segment(p1,p2) = (p1+p2)/2;
geom_inertia_of_point(centroid_of_segment(p1,p2)) + inertia_of_segment(-pdelta/2,pdelta/2)

geom_inertia_of_point(['x,'y,'z])
check_exact(geom_INTERNAL_inertia_of_pnsegment1(['x,'y,'z]), \
            inertia_of_segment(-['x,'y,'z],['x,'y,'z]));


\\ Mg=inertia_of_point((p1+p2)/2)

quit;




\\-----------------------------------------------------------------------------

\\ sqrt(5-12*I)
\\ sqrt((13+5)/2) - sqrt((13-5)/2)*I
\\ abs(z)^2 = 1/4*(Ix + Iy)^2 + Ixy^2
\\ sqrt( (C-A)/2 ) + sqrt( (C+A)/2 )*I
\\ 1/2*sqrt( C^2-A^2 ) +  (C+A)/2 *I
\\ B +  (C+A) *I
\\ (C-A) + B*I
\\ (13+5) + 12*I == 18 + 12*I
\\ (13+5) + 12*I == (3+2*I)*6
\\ 12 + (13-5)*I == 12 + 8*I
\\ 12 + (13-5)*I == (3+2*I)*4

\\ (('C+'A) - 'B*I)^2 == 'A^2 + 2*'A*'C + 'C^2 - 'B^2 - (2*'B*'C + 2*'B*'A)*I
\\   == 2*'A^2 - 2*'A*'C  + (2*'B*'C - 2*'B*'A)*I
\\   == 'A*(2*'A - 2*'C)  + 'B*(2*'C - 2*'A)*I

\\ B = Ixy
\\ A = 1/2*(Ix-Iy)
\\ C = sqrt( 1/4*(Ix-Iy) + Ixy^2 )
\\ Iz*C = sqrt( 1/4*(Ix-Iy) + Ixy^2 ) * (Ix+Iy)
\\      = sqrt( 1/4*(Ix-Iy)*(Ix+Iy)^2 + Ixy^2*(Ix+Iy)^2 )
\\      = sqrt( 1/4*(Ix^2-Iy^2)*(Ix+Iy) + Ixy^2*(Ix+Iy)^2 )
\\      = sqrt( 1/4*(Ix^2-Iy^2)*Iz + Ixy^2*Iz^2 )

geom_inertia_principal_dir(M) =
{
  my(A = geom_inertia_Ix(M) - geom_inertia_Iy(M),
     B = -2*geom_inertia_Ixy(M));
  print("A=",A);
  print("B=",B);
  [geom_INTERNAL_hypot(A,B) + A, B];
}

\\ A=10-4=6
\\ B=4
\\ C=5
\\ C-A = 2
{
  my(PM = geom_inertia_of_point(1),
     M = 26*geom_inertia_rotate_xydir(PM, 3+2*I),
     dir =geom_inertia_principal_dir(M));
  print("PM ",PM);
  print("M ",M);
  print("z ",geom_inertia_arctan_z(M));
  print("z sqrt ",sqrt(geom_inertia_arctan_z(M)));
  print("dir ",dir);
  check_exact(geom_inertia_rotate_xydir(M,dir),
              [26,0,0; 0,0,0; 0,0,26]);
  check_exact(dir, [16, 24]);
}
{
  my(PM = geom_inertia_of_point(1),
     M = 26*geom_inertia_rotate_xydir(PM, 3-2*I),
     dir =geom_inertia_principal_dir(M));
  print("PM ",PM);
  print("M ",M);
  print("z ",geom_inertia_arctan_z(M));
  print("z sqrt ",sqrt(geom_inertia_arctan_z(M)));
  print("dir ",dir);
  check_exact(geom_inertia_rotate_xydir(M,dir),
              [26,0,0; 0,0,0; 0,0,26]);
  check_exact(dir, [16,-24]);
}


quit;

\\-----------------------------------------------------------------------------
\\ 

z1 = a+b*I;
z2 = c+d*I;
z3 = e+f*I;
G = (z1+z2+z3)/3;
area = geom_area_of_triangle(z1,z2,z3);

{
  p = (geom_inertia_of_triangle(z1-G,z2-G)
       + geom_inertia_of_triangle(z2-G,z3-G)
       + geom_inertia_of_triangle(z3-G,z1-G)
      ) / area
}
{
  q = (geom_inertia_of_polygon([z1,z2,z3])
      ) / area
}
print("diff");
geom_inertia_Ix(p)-geom_inertia_Ix(q)

quit();

\\-----------------------------------------------------------------------------
vector_offset(v,z) = apply(x->x+z, v);

v = vector_offset([0,1,1+I,I], -1/2 - 1/2*I);
show(z) =
{
  my(m = geom_inertia_of_polygon(vector_offset(v,z))  * 12);
  print(geom_inertia_Ix(m),"  ",
        geom_inertia_Iy(m),"  ",
        geom_inertia_Ixy(m),"  ",
        geom_inertia_Iz(m),"  ");
}
show(0)
show(1)
show(2)
show(3)
show(4)

\\ Ix = 1
\\ Iy = 12*x^2 + 1
\\ Ixy = 0
\\ Iz = 12*x^2 + 2
vector(10,x,x--; geom_inertia_Ixy(geom_inertia_of_polygon(vector_offset(v,x)) * 12))
quit();



\\-----------------------------------------------------------------------------

z1 = 3;
z2 = 1/4444*I;
m = geom_inertia_of_triangle(z1,z2)

geom_inertia_arctan_z(m) =  (geom_inertia_Ix(m) - geom_inertia_Iy(m))/2   + geom_inertia_Ixy(m)*I;
geom_inertia_arctan_z(m)
geom_inertia_arctan_fraction(m)
print("max ",geom_inertia_principal_xyangle(m));

geom_inertia_arctan_z(m) =  (geom_inertia_Iy(m) - geom_inertia_Ix(m))/2   - geom_inertia_Ixy(m)*I;
geom_inertia_arctan_z(m)
geom_inertia_arctan_fraction(m)
print("max ",geom_inertia_principal_xyangle(m));

quit();


\\-----------------------------------------------------------------------------

\\  [ cos,  sin,  0 ]   [ x^2,    -2*x*y, 0 ]   [ cos, -sin,  0 ]
\\  [ -sin, cos,  0 ] * [ -2*x*y,    y^2, 0 ] * [ sin, cos,  0 ]
\\  [    0,   0,  1 ]   [ 0,      0,      1 ]   [    0,   0,  1 ]
\\
\\ = [xx*c^2 + 2*xy*s*c + yy*s^2         xy*c^2 + (-xx + yy)*s*c - xy*s^2  0]
\\   [xy*c^2 + -xx + yy)*s*c - xy*s^2     yy*c^2 - 2*xy*s*c + xx*s^2      0]
\\   [                               0                                0 1]

rot = [c,s,0;-s,c,0;0,0,1 ]
in = [x*x, -x*y, 0; -x*y, y*y, 0; 0,0,zz];
in_rot = rot * in * mattranspose(rot)

\\ Ika(k,a) = Ix(k)*cos(a)^2 + Ixy(k)*cos(a)*sin(a) + Iy(k)*sin(a)^2;

xr = c*x - s*y
yr = s*x + c*y

ir = [xr*xr, -xr*yr, 0; -xr*yr, yr*yr, 0; 0,0,zz]
ir == in_rot
ir - in_rot
\\ x'^2 = c^2*x^2 - 2*c*s*x*y + s^2*y^2
\\ x'*y' = s*c*x^2 + (c^2 - s^2)*x*y - s*c*y^2


quit();

\\-----------------------------------------------------------------------------

sqrt2 = geom_sqrt_as_quad(2);
check_exact(sqrt2^2, 2);

sqrt2i = geom_sqrt_as_quad(-2);
check_exact(sqrt2i^2, -2);

angle = 60.12821858176463241930427270 * Pi/180;
m_rot_plus45 = {[cos(angle), -sin(angle), 0;
                 sin(angle), cos(angle), 0;
                 0, 0, 1]};
m_rot_minus45 = {[cos(angle), sin(angle), 0;
                 -sin(angle), cos(angle), 0;
                 0, 0, 1]};

m_other_plus45 = {[cos(Pi/2-angle), -sin(Pi/2-angle), 0;
                   sin(Pi/2-angle), cos(Pi/2-angle), 0;
                   0, 0, 1]};
m_other_minus45 = {[cos(Pi/2-angle), sin(Pi/2-angle), 0;
                    -sin(Pi/2-angle), cos(Pi/2-angle), 0;
                    0, 0, 1]};

check_nearly(mattranspose(m_rot_plus45), m_rot_plus45^-1, "trans = inv");
check_nearly(m_rot_plus45 * m_rot_minus45, matid(3), "identity");

check_nearly(m_rot_plus45 * [1,0,0]~, [cos(angle), sin(angle), 0]~, "X rot");
check_nearly(m_rot_plus45 * [0,1,0]~, [-sin(angle), cos(angle), 0]~, "Y rot");

\\ m_rot_plus45 = {[1/sqrt(2), -1/sqrt(2), 0;
\\                  1/sqrt(2), 1/sqrt(2), 0;
\\                  0, 0, 1]};
\\ m_rot_plus45
\\ type(m_rot_plus45)
\\ matrix_sum(m_rot_plus45)

\\ check_nearly(m_rot_plus45 * [1,0,0]~, [1/sqrt(2),1/sqrt(2),0]~, "X rot 45");
\\ check_nearly(m_rot_plus45 * [0,1,0]~, [-1/sqrt(2),1/sqrt(2),0]~, "Y rot 45");
check_nearly(matdet(m_rot_plus45), 1, "det")

b = cos(-angle) + I*sin(-angle);
\\ (1+I)/sqrt(2);
check_nearly(abs(b), 1, "b abs");

z1 = 3;
z2 = 4*I;

print("in");
in = geom_inertia_of_triangle(z1,z2)
print("axis angles");
geom_inertia_principal_xyangle(in) *180/Pi
geom_inertia_principal_xyangle(in) *180/Pi + 90

print("in45");
in45 = geom_inertia_of_triangle(z1*b, z2*b)

print("products");
m_rot_plus45^-1 * in * m_rot_plus45
m_rot_plus45 * in * m_rot_plus45^-1
m_rot_minus45^-1 * in * m_rot_minus45
m_rot_minus45 * in * m_rot_minus45^-1

print("other");
m_other_plus45^-1 * in * m_other_plus45
m_other_plus45 * in * m_other_plus45^-1
m_other_minus45^-1 * in * m_other_minus45
m_other_minus45 * in * m_other_minus45^-1

quit();

  check_exact(mattranspose(m_rot_plus45) * in * m_rot_plus45, in45, "trans first");
  check_exact(m_rot_plus45 * in * mattranspose(m_rot_plus45), in45, "trans second");
  \\ check_exact(m_rot_plus45^-1 * in * m_rot_plus45, in45, "inv first");
  \\ check_exact(m_rot_plus45 * in * m_rot_plus45^-1, in45, "inv second");

m_rot_plus45 * in * m_rot_plus45^-1;
m_rot_plus45^-1 * in * m_rot_plus45;

}

\\ m_trans_xy * in * m_trans_xy == geom_inertia_of_triangle(4, 3*I)


